package dev.fyloz.trial.colorrecipesexplorer.web.controller;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.FileUtils;
import dev.fyloz.trial.colorrecipesexplorer.xlsx.XlsxExporter;
import org.commonmark.node.Node;
import org.commonmark.parser.Parser;
import org.commonmark.renderer.html.HtmlRenderer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ResourceLoader;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.*;
import static dev.fyloz.trial.colorrecipesexplorer.web.StringBank.MATERIALS;

@Controller
public class OthersController {

    private RecipeService recipeService;
    private MaterialService materialService;
    private MixTypeService mixTypeService;
    private MixService mixService;
    private ResourceLoader resourceLoader;

    @Autowired
    public OthersController(RecipeService recipeService, MaterialService materialService, MixTypeService mixTypeService, MixService mixService, ResourceLoader resourceLoader) {
        this.recipeService = recipeService;
        this.materialService = materialService;
        this.mixTypeService = mixTypeService;
        this.mixService = mixService;
        this.resourceLoader = resourceLoader;
    }

    @GetMapping(CLOSE_TAB)
    public ModelAndView closeTab() {
        return new ModelResponseBuilder(CLOSE_TAB).build();
    }

    @GetMapping(MATERIAL_SELECTOR_MIX)
    public ModelAndView getMaterialSelectorFragment(@PathVariable int recipeID, @PathVariable int mixID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(MATERIAL_SELECTOR_FRAGMENT);

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (!optionalRecipe.isPresent()) {
            return modelResponseBuilder
                    .withView("")
                    .build();
        }

        Optional<Mix> optionalMix = mixService.getByID(mixID);
        boolean mixExist = optionalMix.isPresent();

        List<MixType> associatedMixTypes = recipeService.getAssociatedMixesTypes(optionalRecipe.get());
        // Récupère seulement les produits qui ne sont pas des types de mélange OU que ce type existe dans la recette
        List<Material> materials = materialService
                .getAll()
                .stream()
                .filter(m -> !m.isMixType() || (!mixExist || !m.equals(optionalMix.get().getMixType().getMaterial())) && associatedMixTypes.contains(mixTypeService.getByMaterial(m).get()))
                .sorted(Comparator.comparing(Material::getMaterialCode))
                .sorted(Comparator.comparing(m -> m.getMaterialType().getMaterialTypeName()))
                .collect(Collectors.toList());

        return modelResponseBuilder
                .addAttribute(MATERIALS, materials)
                .build();
    }

    @GetMapping(TOUCHUP)
    public ModelAndView getTouchUpPdf() {
        return new ModelResponseBuilder().withRedirect("pdf/touchup.pdf").build();
    }

    @GetMapping(RECIPE_XLS)
    public ResponseEntity<byte[]> getXlsForRecipe(HttpServletRequest request, @PathVariable int recipeID) {
        HttpHeaders headers = new HttpHeaders();

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (!optionalRecipe.isPresent()) {
            headers.add(HttpHeaders.LOCATION, request.getHeader("referer"));
            return new ResponseEntity<>(headers, HttpStatus.FOUND);
        }

        byte[] recipeXLS = new XlsxExporter().generate(optionalRecipe.get());
        if (recipeXLS.length <= 0) return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(recipeXLS.length)
                .contentType(MediaType.parseMediaType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"))
                .body(recipeXLS);
    }

    @GetMapping(value = ALL_RECIPES_XLS, produces = "application/zip")
    public ResponseEntity<byte[]> getAllXls() throws IOException {
        HttpHeaders headers = new HttpHeaders();

        ColorRecipesExplorerApplication.LOGGER.info("Exportation de toutes les couleurs en XLS");

        ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
        ZipOutputStream out = new ZipOutputStream(byteOS);

        Collection<Recipe> recipes = recipeService.getAll();
        for (Recipe recipe : recipes) {
            byte[] recipeXLS = new XlsxExporter().generate(recipe);
            if (recipeXLS.length <= 0) return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();

            out.putNextEntry(new ZipEntry(String.format("%s_%s.xlsx", recipe.getCompany().getCompanyName(), recipe.getRecipeCode())));
            out.write(recipeXLS, 0, recipeXLS.length);
            out.closeEntry();
        }

        out.close();

        byte[] zipContent = byteOS.toByteArray();
        byteOS.close();

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(zipContent.length)
                .contentType(MediaType.parseMediaType("application/zip"))
                .body(zipContent);
    }

    @GetMapping(value = UPDATES)
    public ModelAndView showUpdates() {
        return new ModelResponseBuilder(UPDATES).build();
    }

    @GetMapping(value = UPDATES_GET, produces = MediaType.TEXT_HTML_VALUE)
    @ResponseBody
    public ResponseEntity<String> getUpdates() throws IOException {
        String fileContent = FileUtils.readClasspathFile(resourceLoader, "classpath:updates.md");
        if (fileContent == null) return ResponseEntity.status(HttpStatus.NOT_FOUND).build();

        Parser parser = Parser.builder().build();
        Node document = parser.parse(fileContent);
        HtmlRenderer renderer = HtmlRenderer.builder().build();

        return new ResponseEntity<>(renderer.render(document), HttpStatus.OK);
    }
}
