package dev.fyloz.trial.colorrecipesexplorer.web.controller.creators;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.CREATOR_MATERIAL;

@Controller
public class MaterialCreatorController {

    private MaterialService materialService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialCreatorController(MaterialService materialService, MaterialTypeService materialTypeService) {
        this.materialService = materialService;
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(CREATOR_MATERIAL)
    public ModelAndView showCreationPage(ModelAndView model, Material material) {
        return new ModelResponseBuilder(model)
                .withView(CREATOR_MATERIAL)
                .addResponseData(ResponseDataType.MATERIAL, material != null ? material : new Material())
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    /**
     * Permet à l'utilisateur de créer un produit.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à effectuer cette action
     * - Le produit existe déjà
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code de produit du produit créé
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param material Le produit à créer
     * @param simdut   Le fichier SIMDUT du produit (optionnel)
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_MATERIAL, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ModelAndView create(@Valid Material material, MultipartFile simdut) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        if (!materialService.exists(material)) {
            Optional<Material> savedMaterial = materialService.save(material);

            if (savedMaterial.isPresent()) {
                material = savedMaterial.get();

                modelResponseBuilder.addResponseData(ResponseDataType.MATERIAL_CODE, material.getMaterialCode());

                if (simdut.getSize() > 0 && !materialService.addSimdut(simdut, material)) {
                    modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING_SIMDUT);
                }

                return showCreationPage(modelResponseBuilder
                        .addResponseData(ResponseDataType.MATERIAL_CODE, material.getMaterialCode())
                        .build(), null);
            } else {
                modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING);
            }
        } else {
            modelResponseBuilder.addResponseCode(ResponseCode.MATERIAL_ALREADY_EXIST, material.getMaterialCode());
        }

        return showCreationPage(modelResponseBuilder.build(), material);
    }
}
