package dev.fyloz.trial.colorrecipesexplorer.web.controller.creators;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.MixCreationFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.*;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.ControllerUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.*;

@Controller
public class MixCreatorController {

    private MixService mixService;
    private RecipeService recipeService;
    private MaterialService materialService;
    private MixTypeService mixTypeService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MixCreatorController(MixService mixService, RecipeService recipeService, MaterialService materialService, MixTypeService mixTypeService, MaterialTypeService materialTypeService) {
        this.mixService = mixService;
        this.recipeService = recipeService;
        this.materialService = materialService;
        this.mixTypeService = mixTypeService;
        this.materialTypeService = materialTypeService;
    }

    /**
     * Affiche la page de création d'un mélange.
     * Cette méthode requiert l'identifiant d'une recette dans l'URL.
     *
     * @param model    Le Model injecté par Thymeleaf
     * @param recipeID L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_MIX_SPECIFIC)
    public ModelAndView showCreationPage(ModelAndView model, @PathVariable int recipeID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(model)
                .withView(CREATOR_MIX);

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (!optionalRecipe.isPresent()) {
            return modelResponseBuilder
                    .withView(ControllerUtils.redirect(EDITOR_RECIPE))
                    .build();
        }

        Recipe recipe = optionalRecipe.get();

        List<MixType> associatedMixTypes = recipeService.getAssociatedMixesTypes(recipe);
        // Récupère seulement les produits qui ne sont pas des types de mélange OU que ce type existe dans la recette
        List<Material> materials = materialService
                .getAll()
                .stream()
                .filter(m -> !m.isMixType() || associatedMixTypes.contains(mixTypeService.getByMaterial(m).get()))
                .sorted(Comparator.comparing(Material::getMaterialCode))
                .sorted(Comparator.comparing(m -> m.getMaterialType().getMaterialTypeName()))
                .collect(Collectors.toList());

        ModelResponseBuilder responseBuilder = modelResponseBuilder
                .addResponseData(ResponseDataType.RECIPE, recipe)
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .addAttribute("materialsJson", materialService.asJson(materials));

        if (materialService.getAll().isEmpty()) {
            responseBuilder.addResponseCode(ResponseCode.NO_MATERIAL)
                    .addResponseData(ResponseDataType.BLOCK_BUTTON, true);
        }

        return responseBuilder.build();
    }

    /**
     * Permet à l'utilisateur de créer un mélange.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Un des produits n'existe pas
     * - Une erreur est survenue lors de la sauvegarde du type de mélange
     * - Une erreur est survenue lors de la sauvegarde du mélange
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param form La formulaire du mélange entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_MIX, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView createMix(@ModelAttribute @Valid MixCreationFormDto formDto) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        Recipe recipe = formDto.getRecipe();
        if (recipe == null) return modelResponseBuilder
                .withRedirect(EDITOR_RECIPE)
                .addResponseCode(ResponseCode.RECIPE_NOT_FOUND_NO_PARAMS)
                .build();

        ModelResponseBuilder mixCreationResponse = mixService.create(formDto, recipe);
        if (mixCreationResponse != null)
            return showCreationPage(mixCreationResponse.build(), formDto.getRecipe().getRecipeID());

        return modelResponseBuilder
                .withRedirect(EDITOR_RECIPE_SPECIFIC, recipe.getRecipeID())
                .build();
    }
}
