package dev.fyloz.trial.colorrecipesexplorer.web.controller.editors;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.MixCreationFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.*;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.ControllerUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.*;
import static dev.fyloz.trial.colorrecipesexplorer.web.StringBank.*;

@Controller
public class MixEditorController {

    private MixService mixService;
    private MaterialService materialService;
    private RecipeService recipeService;
    private MixTypeService mixTypeService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MixEditorController(MixService mixService, MaterialService materialService, RecipeService recipeService, MixTypeService mixTypeService, MaterialTypeService materialTypeService) {
        this.mixService = mixService;
        this.materialService = materialService;
        this.recipeService = recipeService;
        this.mixTypeService = mixTypeService;
        this.materialTypeService = materialTypeService;
    }

    /**
     * Affiche la page d'édition d'un mélange.
     * Cette méthode requiert l'identifiant du mélange à modifier dans l'URL.
     *
     * @param model Le Model injecté par Thymeleaf
     * @param mixID L'identifiant du mélange à modifier
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_MIX_SPECIFIC)
    public ModelAndView showPage(ModelAndView model, @PathVariable int mixID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(model)
                .withView(EDITOR_MIX_SPECIFIC.replaceAll("/\\{" + MIX_ID + "}", ""));

        Optional<Mix> optionalMix = mixService.getByID(mixID);
        if (!optionalMix.isPresent()) {
            return modelResponseBuilder
                    .withView(ControllerUtils.redirect(EDITOR_RECIPE))
                    .build();
        }

        Mix mix = optionalMix.get();
        List<MixType> associatedMixTypes = recipeService.getAssociatedMixesTypes(mix.getRecipe());
        // Récupère seulement les produits qui ne sont pas des types de mélange OU que ce type existe dans la recette
        List<Material> materials = materialService
                .getAll()
                .stream()
                .filter(m -> !m.isMixType() || (!m.equals(mix.getMixType().getMaterial()) && associatedMixTypes.contains(mixTypeService.getByMaterial(m).get())))
                .sorted(Comparator.comparing(Material::getMaterialCode))
                .sorted(Comparator.comparing(m -> m.getMaterialType().getMaterialTypeName()))
                .collect(Collectors.toList());

        return modelResponseBuilder
                .addResponseData(ResponseDataType.MIX, mix)
                .addResponseData(ResponseDataType.RECIPE, mix.getRecipe())
                .addAttribute(MATERIAL_TYPE, mix.getMixType().getMaterial().getMaterialType())
                .addAttribute(MATERIAL_TYPES, materialTypeService.getAll())
                .addAttribute("mixJson", materialService.asJson(mix))
                .addAttribute("materialsJson", materialService.asJson(materials))
                .build();
    }

    /**
     * Permet à l'utilisateur de modifier un mélange.
     * <p>
     * La mise à jour échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Le mélange n'existe pas
     * - Un des produits n'existe pas
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param form Le formulaire du mélange entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_MIX, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView saveMix(@ModelAttribute @Valid MixCreationFormDto formDto, int mixID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        Optional<Mix> optionalMix = mixService.getByID(mixID);
        if (!optionalMix.isPresent()) {
            modelResponseBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, mixID);

            return showPage(modelResponseBuilder.build(), mixID);
        }

        Mix mix = optionalMix.get();
        modelResponseBuilder.withRedirect(EDITOR_RECIPE_SPECIFIC, mix.getRecipe().getRecipeID());

        ModelResponseBuilder editResult = mixService.edit(mix, formDto);
        if (editResult != null) {
            return showPage(
                    modelResponseBuilder
                            .addResponseCode(ResponseCode.ERROR_SAVING)
                            .build(),
                    mixID);
        }

        return modelResponseBuilder.build();
    }

    @GetMapping(REMOVER_MIX_SPECIFIC)
    public ModelAndView deleteMix(@PathVariable int mixID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder("");

        Optional<Mix> optionalMix = mixService.getByID(mixID);
        if (!optionalMix.isPresent()) {
            return showPage(modelResponseBuilder
                            .addResponseCode(ResponseCode.MIX_NOT_FOUND, mixID)
                            .build(),
                    mixID);
        }

        Mix mix = optionalMix.get();
        modelResponseBuilder.withView(ControllerUtils.redirect(EDITOR_RECIPE_SPECIFIC.replace("{recipeID}", String.valueOf(mix.getRecipe().getRecipeID()))));

        if (!mixService.deleteMix(mix)) {
            modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING);

            return showPage(modelResponseBuilder.build(), mixID);
        }

        return modelResponseBuilder.build();
    }
}
